/*
    file: mGNFAEnum.cpp 
    Project: miniGrail
    License: LGPLv2.1 http://www.gnu.org/licenses/lgpl-2.1.html

    Created by: by Rui Zhou
    Maintainer: Cezar Campeanu <ccampeanu@upei.ca>


 * This algorithm is imported from Grail+
 * This algorithm enumerate the words in an NFA's language in
 * Lexicographical order
 *
 * Little change is made, the program stops when reaches any
 * of the situations:
 * 1. run out of words
 * 2. go over the max length
 * 3. reach the max number
 */

/*
 * parameters:
 * The nfa to enumerate from
 * The Array to hold the result
 * The maximum number of strings to enumerate
 * The maximum length of the string to enumerate
 */
int mGNFAEnum(mGNFA& nfa, mGArray<mGString>& words, int max_num, int max_length)
{
	//TODO: DEBUG TRIGGER

	int i;
	int j;
	int k;
	int m;
	mGNFA cur;
	mGString r;
	mGSet<mGState> temp;
	mGArray<mGSet<mGState> > dest;
	mGArray<mGSet<mGState> > next_dest;
	mGArray<mGString> partial;
	mGArray<mGString> next_partial;

	temp.clear();
	dest += temp;

	// check to make sure some final states are reachable
	nfa.reachable_states(dest[0]);
	temp.intersect(dest[0], nfa.getStartStates());
	if (temp.size() == 0)
		return 0;

	// minimize the machine
	nfa.reachable();
	nfa.reverse();
	nfa.reachable();
	nfa.reverse();

	// begin with start states
	temp = nfa.getStartStates();
	dest.clear();
	dest += temp;
	r.clear();
	partial += r;

	// develop word fragments until we have enough
	for (;;)
	{
		// extract new words from fragment mGArray
		for (i = 0; i < partial.size(); ++i)
		{
			for (j = 0; j < dest[i].size(); ++j)
				if (nfa.getFinalStates().member(dest[i][j]) >= 0)
				{
					words += partial[i];

					if (partial[i].size() > max_length)
						return words.size();

					break;
				}
		}

		if (words.size() >= max_num)
		{

			return (words.size());
		}

		// develop new fragments
		next_dest.clear();
		next_partial.clear();

		for (j = 0; j < dest.size(); ++j)
		{
			for (i = 0; i < dest[j].size(); ++i)
			{
				if (dest[j].member(dest[j][i]) < 0)
					continue;

				nfa.select(dest[j][i], SOURCE, cur);

				// if no arcs, continue
				if (cur.size() == 0)
					continue;

				// else add new fragments
				for (k = 0; k < cur.size(); ++k)
				{
					r = partial[j];
					r += cur[k].getLabel();

					// don't add any fragments that
					// are already present
					if ((m = next_partial.member(r)) >= 0)
					{
						next_dest[m] += cur[k].getEnd();
						continue;
					}

					next_partial += r;
					temp.clear();
					temp += cur[k].getEnd();
					next_dest += temp;
				}
			}
		}

		// reset the wave
		if (next_dest.size() == 0)
		{

			return (words.size());
		} // added by ray ,sort before output

		dest = next_dest;
		partial = next_partial;
	}

	return -1;
}
