/*
    file: mGNFAtoDFA_Set.cpp 
    Project: miniGrail
    License: LGPLv2.1 http://www.gnu.org/licenses/lgpl-2.1.html

    Created by: by Rui Zhou
    Maintainer: Cezar Campeanu <ccampeanu@upei.ca>

 

  This use the very basic set algorithm to transform an NFA into a DFA
*/

void mGNFAtoDFA_Set(mGNFA& nfa, mGDFA& dfa) //question  why can not const?
{
//DEBUG ?
	bool debug = false;
	if (DEBUG_TRIGGERED(DEBUG_mGNFAtoDFA_Set) || DEBUG_TRIGGERED(DEBUG_ALL))
	{
		debug = true;
		std::cout << "Debug output for mGNFAtoDFA_Set(NFA, DFA) is enabled."
				<< std::endl;

	}

//clear the DFA to output
	dfa.clear();
//attach
	nfa.attach();

//gather the labels in the NFA
	mGSet<int> alphabets;
	nfa.gatherLabels(alphabets);
	if (debug)
	{
		std::cout << "Gathered alphabets" << alphabets << std::endl;
	}

// the sets that will be the DFA states
	mGArray<mGSet<mGState> > setsToProcess;
	int index = 0; // current index in the sets to process

	setsToProcess += nfa.getStartStates();

	if (debug)
	{
		std::cout << "The start states size:" << setsToProcess[0].size()
				<< std::endl;
	}

//calculate
	while (index < setsToProcess.size())
	{
		if (debug)
		{
			std::cout << "The setsToProcess: " << setsToProcess << std::endl;
			std::cout << "Set To Process: " << setsToProcess[index]
					<< std::endl;
		}

///get all involved transitions
		mGSet<mGTransition> transitionsInvolved;

		for (int a = 0; a < setsToProcess[index].size(); a++) //look at  each set of states
		{
			transitionsInvolved += nfa.getTransitionsStartFrom(
					(setsToProcess[index])[a]);
		}

		if (debug)
		{
			std::cout << "transitions involved: " << transitionsInvolved
					<< std::endl;
		}
//store them by labels
		mGMap<int, mGSet<mGState> > labelSets;

		for (int a = 0; a < transitionsInvolved.size(); a++)
		{
			labelSets[transitionsInvolved[a].getLabel()] +=
					transitionsInvolved[a].getEnd();
		}

//now , look at the dictionary (map), add new Sets to setsToProcess as well as
//add transitions to DFA
		for (mGMap<int, mGSet<mGState> >::mGMapItator it = labelSets.begin();
				it != labelSets.end(); it++)
		{
			//add new set to process if needed
			if (setsToProcess.member((*it).second) < 0)
			{
				setsToProcess += (*it).second;
			}
			//add transitions
			mGTransition t(index, (*it).first,
					setsToProcess.member((*it).second));
			dfa.addTransition(t);
		}

		index++;
	}
//now do the start and end state
	mGState s(0);
	dfa.addStart(0);
	mGSet<mGState> originalFinalSet = nfa.getFinalStates();

	if (debug)
	{
		std::cout << "Original final set" << originalFinalSet << std::endl;
	}

	for (int a = 0; a < setsToProcess.size(); a++)
	{
		mGSet<mGState> inter;
		mGArray<mGState>::getIntersection(originalFinalSet, setsToProcess[a],
				inter);

		if (inter.size() > 0)
			dfa.addFinal(a);

	}

}
