/*
    file: mGInt.cpp 
    Project: miniGrail
    License: LGPLv2.1 http://www.gnu.org/licenses/lgpl-2.1.html

    Created by: by Rui Zhou
    Maintainer: Cezar Campeanu <ccampeanu@upei.ca>


 * Implementation of mGInt
 *
 */

#include <assert.h>
#include <string.h>
#include <sstream>
#include <bitset>
#include <stdio.h>
#include <limits.h>

mGInt::mGInt(const mGInt& c)
{
	size = c.getSize();
	ptr = new char[size];

	memcpy((void*) ptr, (void*) (c.getData()), size);
}

mGInt::mGInt(const mGSet<mGState>& set)
{
// get the biggest number
//which is the digits we need

	int digit_num = 0;

	for (int a = 0; a < set.size(); a++)
	{
		{
			int temp = set[a].getNumber();
			assert(temp>=0);
			// null state or error
			if (temp > digit_num)
			{

				digit_num = temp;

			}

		}

//we new something now!
// the raw memory chunk that stores things
		size = int(digit_num / 8) + 1;
		ptr = new char[size];

		//clean the memory
		//necessary in 32 bit systems somehow
		for (int a = 0; a < size; a++)
		{

			ptr[a] = 0;

		}

//add each number to the  memory chuck

		for (int a = 0; a < set.size(); a++)
		{
			int temp = set[a].getNumber();
			ptr[int(temp / 8)] += 1 << (temp % 8); //be smart

		}

	}
}
mGInt::mGInt(const string& str)
{
	size = str.length();

	ptr = new char[size];

	//clean the memory
	//necessary in 32 bit systems somehow
	for (int a = 0; a < size; a++)
	{

		ptr[a] = 0;

	}
	//must cast it to a c string
	memcpy((void*) ptr, (void*) (str.c_str()), size);

}

// Convent to binary by bitwise and
void mGInt::toBinray(string& str)
{
	str.clear();
	for (int a = size - 1; a >= 0; a--)
	{
		char temp = ptr[a];
		//std::cout<<"."<<temp<<".";

		for (int b = 7; b >= 0; b--)
		{

			if (DIGIT[b] & temp) //be smart
			{
				str += '1';
			}
			else
			{
				str += '0';
			}

		}
		str += " ";
	}
	//std::cout<<std::endl;
	//str += "";
}

//convert to binary with bitset cast
void mGInt::toBitSet(string& str) const
{
	stringstream stream;
	str.clear();
	for (int a = size - 1; a >= 0; a--)
	{
		stream << std::bitset<CHAR_BIT>(char(ptr[a])) << " ";
	}

	str = stream.str();
}

// compare
/*
 * The default string compare does not put length at the first place
 * but mGInt does so , because of efficiency
 */
const int mGInt::compare(const mGInt& c) const
{

	if (size > c.getSize())
	{
		return 1; //bigger if the size is bigger
	}

	if (size < c.getSize())
	{
		return -1; //Smaller if the size are smaller
	}

	// the same size
	{

		return memcmp(ptr, c.getData(),size);

	}

}

