/*
    file: Main.cpp 
    Project: miniGrail
    License: LGPLv2.1 http://www.gnu.org/licenses/lgpl-2.1.html

    Created by: by Rui Zhou
    Maintainer: Cezar Campeanu <ccampeanu@upei.ca>

*/

#include <iostream>
#include <sstream>
#include "../miniGrail.h"

/*
 * General variables
 */
//all the arguments
mGArray<string> args;
//if the -info flag exits
bool show_info = false;
bool specific_debug = false;
/*
 include filters here instead of in miniGrail.h because
 The filters are only used with the main function.
 */
#include "../Filters/Filters.h"

int main(int argc_in, char** argv_in)
{
	//load all the filters
	load_filters();

	// the link name
	string filter_name(argv_in[0]);
	//remove leading "./" if there is one
	if (filter_name.find("./") == 0)
	{
		filter_name = filter_name.substr(2);
		//cout << filter_name;
	}

	//record the rest arguments
	for (int a = 1; a < argc_in; a++)
	{
		args += string(argv_in[a]);

	}

	//the debug flag -d and -da
	if (args.member("-da") >= 0)
	{
		cout << "[Will output all debug info!]"<<endl;
		DEBUG_TRIGGER(DEBUG_ALL);
		args -= "-da";

	}
	else if (args.member("-d") >= 0)
	{
		specific_debug = true;
		args -= "-d";

	}

	//open the info page via browser
	if (args.member("--infoPage") >= 0)
	{
		show_info = true;
		args -= "--infoPage";

		//open the local link and exit
		char* prog[3];
		prog[0] = (char*) "/usr/bin/xdg-open";

		stringstream ss;
		ss << "../Filters/";
		ss << filter_name;
		ss << "/";
		ss << filter_name;
		ss << ".html";

		prog[1] = (char*) (ss.str().c_str());
		prog[2] = (char*) '\0';

		execvp(prog[0], prog);

		exit(0);
	}

	//open the info page via more
	if (args.member("--info") >= 0)
	{
		show_info = true;
		args -= "--info";

		//open the local link and exit
		char* prog[3];
		prog[0] = (char*) "more";

		stringstream ss;
		ss << "../Filters/";
		ss << filter_name;
		ss << "/";
		ss << filter_name;
		ss << ".html";

		prog[1] = (char*) (ss.str().c_str());
		prog[2] = (char*) '\0';

		cout << endl << "You can also try the flag -infoPage to see this html"
				<< " file with your default web browser!" << endl;
		execvp(prog[0], prog);

		exit(0);
	}

	//version flag
	if (args.member("--version") >= 0)
	{
		cout << MG<< "("<<filter_name<<")" << mGVersion << endl;

		exit(0);
	}
	//help flag
	if (args.member("--help") >= 0)
	{
		cout << "Please use -info to see all the help information" << endl;

		exit(0);
	}

	//execute the filter
	(*(mG_filters[filter_name]))();

	return 0;

}
