/*
    file: mGIStream.cpp
    Project: miniGrail
    License: LGPLv2.1 http://www.gnu.org/licenses/lgpl-2.1.html

    Created by: by Rui Zhou
    Maintainer: Cezar Campeanu <ccampeanu@upei.ca>

 *
 * mGIstream
 * to manager the input to theory objects
 *
 *
 */

#include <iostream>
#include <string>
#include <cstdio>
using namespace std;

//mGState
inline istream& operator>>(istream& is, mGState& s)
{
	int i;

	is >> i;
	is.putback(' ');
	s.setNumber(i);
	return is;

}

inline void operator>>(string& str, mGState& s)
{
	char ch = ' ';
	int index = 0;
	string state;
	//get rid of spaces
	while (ch == ' ' && index < str.length())
	{
		ch = str[index];
		index++;
	}
	//get state1
	while (ch != ' ' && index < str.length())
	{
		state += ch;

		ch = str[index];
		index++;
	}
	if (ch != ' ')
		state += ch;

	s.setNumber(atoi(state.c_str()));

}

//mGTransition
inline istream& operator>>(istream& is, mGTransition& t)
{
	int ch;
// skip white space
	while ((ch = is.get()) != EOF)
		if (!isspace((char) ch))
		{
			is.putback(ch);
			break;
		}

	// if nothing, return null transition
	if (is.eof())
	{
		t.null();
		return is;
	}

	string state1, label, state2;

	// input first state
	while ((ch = is.get()) != EOF)
	{
		if ((char) ch == ' ')
			break;
		state1 += (char) ch;
	}

	// skip white space
	while ((ch = is.get()) != EOF)
		if (!isspace((char) ch))
		{
			is.putback(ch);
			break;
		}
	// input label
	while ((ch = is.get()) != EOF)
	{
		if ((char) ch == ' ')
			break;
		label += (char) ch;
	}

	// skip white space
	while ((ch = is.get()) != EOF)
		if (!isspace((char) ch))
		{
			is.putback(ch);
			break;
		}

	// input second state
	while ((ch = is.get()) != EOF)
	{
		if (isspace((char) ch))
			break;
		state2 += (char) ch;
	}

	//note: start and final states should be checked in
	//nfa:operator>>
	//Because they are attributes of a machine

	//set the values
	t.setStart(atoi(state1.c_str()));
	t.setLabel(getSymbolTableIndex(label));
	t.setEnd(atoi(state2.c_str()));

	return is;

}

inline void operator>>(string& str, mGTransition& t)
{
//get the part without spaces
	string state1;
	string label;
	string state2;

	char ch = ' ';
	int index = 0;
	//get rid of spaces
	while (ch == ' ' && index < str.length())
	{
		ch = str[index];
		index++;
	}
	//get state1
	while (ch != ' ' && index < str.length())
	{
		state1 += ch;

		ch = str[index];
		index++;
	}

	//get rid of spaces
	while (ch == ' ' && index < str.length())
	{
		ch = str[index];
		index++;
	}

	//get label
	//get state1
	while (ch != ' ' && index < str.length())
	{
		label += ch;

		ch = str[index];
		index++;
	}

	//get rid of spaces
	while (ch == ' ' && index < str.length())
	{
		ch = str[index];
		index++;
	}
	//get state2
	while (ch != ' ' && index < str.length())
	{
		state2 += ch;

		ch = str[index];
		index++;
	}

	if (ch != ' ')
		state2 += ch;

	//std::cout<<state1<<" "<<label<<" "<<state2<<std::endl;

	t.setStart(atoi(state1.c_str()));
	//std::cout<<";"<<label<<";"<<std::endl;

	t.setLabel(getSymbolTableIndex(label));
	t.setEnd(atoi(state2.c_str()));

}

//MCNFA
inline istream& operator>>(istream& is, mGNFA& nfa)
{
	/*
	 string temp;
	 while (true)
	 {
	 temp.clear();
	 getline(is, temp); //get a line

	 if (temp.find("START") != string::npos)
	 {
	 mGState s;
	 string temps(temp.substr(temp.find("|-") + 2));
	 temps >> s;
	 nfa.addStart(s);

	 }
	 else if (temp.find("FINAL") != string::npos)
	 {
	 mGState s;
	 string temps(temp.substr(0, temp.find("-|")));
	 temps >> s;
	 nfa.addFinal(s);
	 }
	 else if (temp.find(EOF) != string::npos)
	 {
	 break;

	 }
	 else if (temp.length() == 0) /// skip empty line
	 {
	 break;
	 }

	 else
	 {
	 mGTransition t;
	 temp >> t;
	 nfa.getTransitions().disjoint_union(t);
	 //we can use nfa.addtransition(), but that check for duplication, kind slow
	 }

	 }

	 nfa.getTransitions().unique_array();

	 //nfa.unqiueTransitionsByAttach();

	 */

	//read in the finite machine, using a finite machine
	string state1("");
	string label("");
	string state2("");
	int current_state = 0; //initial state

	char c;
	while (is.good())
	{
		c = is.get();
		//cout << current_state;
		//cout << c << endl;

		//for comments

		switch (current_state)
		{
		case -3:
			switch (c)
			{
			case '#':
				current_state = 0;

				break;
			default:

				//ignore some token
				break;
			}
			break;

		case 0:
			switch (c)
			{

			case '(':
			case 'S':
			case 's':
				current_state++;
				break;
			case '0':
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':
				current_state = 3;
				state1 += c;
				break;
			case '#':
				current_state = -3;
				break;
			default:
				//ignore some token
				break;
			}
			break;

		case 1:
			switch (c)
			{
			case 'S':
			case 's':
			case 'T':
			case 't':
			case 'A':
			case 'a':
			case 'R':
			case 'r':
			case ')':
			case '|':
			case '-':
			case ' ':
			case '	':
				//LOL
				break;
			case '0':
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':
				current_state++;
				state1 += c;
				break;
			default:
				current_state = -1; //error
				break;
			}
			break;
		case 2:
			switch (c)
			{

			case '0':
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':

				state1 += c;
				break;

			default:

				nfa.addStart(atoi(state1.c_str()));
				state1.clear();
				current_state = 0;

				break;

			}
			break;
		case 3:
			switch (c)
			{

			case '0':
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':

				state1 += c;
				break;
			case '|':
			case '-':
				nfa.addFinal(atoi(state1.c_str()));
				state1.clear();
				current_state++;
				break;
			case ' ':
			case '	':
				current_state = 5;
				break;

			default:
				current_state = -1;
				break;

			}
			break;

		case 4:
			switch (c)
			{
			case 'F':
			case 'f':
			case 'I':
			case 'i':
			case 'N':
			case 'n':
			case 'A':
			case 'a':
			case 'L':
			case 'l':
			case '(':
			case ')':
			case '|':
			case '-':
			case ' ':
			case '	':
				break;

			default:
				current_state = 0;
				break;
			}
			break;

		case 5:
			switch (c)
			{

			case '|':
			case '-':
				nfa.addFinal(atoi(state1.c_str()));
				state1.clear();
				current_state = 4;
				break;

			case ' ':
			case '	':
				break;

			default:
				label += c;
				current_state++;
				break;

			}
			break;

		case 6:
			switch (c)
			{

			case ' ':
			case '	':

				current_state++;
				break;

			default:
				label += c;

				break;

			}
			break;

		case 7:
			switch (c)
			{
			case '0':
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':
				state2 += c;
				break;
			case ' ':
			case '	':
				break;
			default:
				mGTransition temp(atoi(state1.c_str()),
						getSymbolTableIndex(label), atoi(state2.c_str()));
				nfa.getTransitions().disjoint_union(temp);
				state1.clear();
				label.clear();
				state2.clear();
				current_state = 0;
				break;
			}
			break;

		}

		if (current_state == -1)
		{
			cout << "Input error at token [" << c << "]" << endl;
			exit(1);
		}

	}

//	nfa.getTransitions().unique_array();
	return is;

}

//MCDFCA
/*
 * The input finite machine is basically same as
 * NFA, but an extra loop for l
 *
 *
 */
inline istream& operator>>(istream& is, mGDFCA& dfca)
{
	string length("");
	string state1("");
	string label("");
	string state2("");
	int current_state = 0; //initial state

	char c;
	while (is.good())
	{
		c = is.get();
		//cout << current_state;
		//cout << c << endl;
		switch (current_state)
		{
		//for l
		case -2:
			switch (c)
			{
			case 'l':
			case 'L':
			case ' ':
			case '	':
			case '=':
				break;
			case '0':
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':
				length += c;

				break;
			default:
				dfca.setL(atoi(length.c_str()));
				current_state = 0;
				//ignore some token
				break;
			}
			break;

			//for comments

		case -3:
			switch (c)
			{
			case '#':
				current_state = 0;

				break;
			default:

				//ignore some token
				break;
			}
			break;

		case 0:
			switch (c)
			{

			case '(':
			case 'S':
			case 's':
				current_state++;
				break;
			case '0':
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':
				current_state = 3;
				state1 += c;
				break;

			case 'l':
			case 'L':
				current_state = -2;
				break;

			case '#':
				current_state = -3;
				break;

			default:
				//ignore some token
				break;
			}
			break;

		case 1:
			switch (c)
			{
			case 'S':
			case 's':
			case 'T':
			case 't':
			case 'A':
			case 'a':
			case 'R':
			case 'r':
			case ')':
			case '|':
			case '-':
			case ' ':
			case '	':
				//LOL
				break;
			case '0':
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':
				current_state++;
				state1 += c;
				break;
			default:
				current_state = -1; //error
				break;
			}
			break;
		case 2:
			switch (c)
			{

			case '0':
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':

				state1 += c;
				break;

			default:

				dfca.addStart(atoi(state1.c_str()));
				state1.clear();
				current_state = 0;

				break;

			}
			break;
		case 3:
			switch (c)
			{

			case '0':
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':

				state1 += c;
				break;
			case '|':
			case '-':
				dfca.addFinal(atoi(state1.c_str()));
				state1.clear();
				current_state++;
				break;
			case ' ':
			case '	':
				current_state = 5;
				break;

			default:
				current_state = -1;
				break;

			}
			break;

		case 4:
			switch (c)
			{
			case 'F':
			case 'f':
			case 'I':
			case 'i':
			case 'N':
			case 'n':
			case 'A':
			case 'a':
			case 'L':
			case 'l':
			case '(':
			case ')':
			case '|':
			case '-':
			case ' ':
			case '	':
				break;

			default:
				current_state = 0;
				break;
			}
			break;

		case 5:
			switch (c)
			{

			case '|':
			case '-':
				dfca.addFinal(atoi(state1.c_str()));
				state1.clear();
				current_state = 4;
				break;

			case ' ':
			case '	':
				break;

			default:
				label += c;
				current_state++;
				break;

			}
			break;

		case 6:
			switch (c)
			{

			case ' ':
			case '	':

				current_state++;
				break;

			default:
				label += c;

				break;

			}
			break;

		case 7:
			switch (c)
			{
			case '0':
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':
				state2 += c;
				break;
			case ' ':
			case '	':
				break;
			default:
				mGTransition temp(atoi(state1.c_str()),
						getSymbolTableIndex(label), atoi(state2.c_str()));
				dfca.getTransitions().disjoint_union(temp);
				state1.clear();
				label.clear();
				state2.clear();
				current_state = 0;
				break;
			}
			break;

		}

		if (current_state == -1)
		{
			cout << "Input error at token [" << c << "]" << endl;
			exit(1);
		}

	}

//	nfa.getTransitions().unique_array();
	return is;

}

/// get a mGString
inline istream& operator>>(istream& is, mGString& str)
{

	string source;
	getline(is, source);
	string temp;
	int index = 0;
	int length = source.size();
	temp.clear();
	//std::cout<<"get courcer"<<source;
	while (index < length)
	{
		if (!(source[index] == ' '))
		{
			temp += source[index];
			//std::cout<<"temp"<<temp<<std::endl;
		}
		else
		{ //std::cout<<"add"<<temp;
			if (temp.size() > 0)
			{
				//std::cout<<"add"<<temp<<"to str";
				str += getSymbolTableIndex(temp);
				temp.clear();
			}
		}
		index++;
	}
	if (temp.size() > 0)
	{
		//std::cout<<"add"<<temp<<"to str";
		str += getSymbolTableIndex(temp);
		temp.clear();
	}

	return is;
}

/// get a mGString
inline void operator>>(string& source, mGString& str)
{

	string temp;
	int index = 0;
	int length = source.size();
	temp.clear();
	while (index < length)
	{
		if (!(source[index] == ' '))
		{
			temp += source[index];
			//std::cout<<"temp"<<temp<<std::endl;
		}
		else
		{ //std::cout<<"add"<<temp;
			if (temp.size() > 0)
			{
				//std::cout<<"add"<<temp<<"to str";
				str += getSymbolTableIndex(temp);
				temp.clear();
			}
		}
		index++;
	}
	if (temp.size() > 0)
	{
		//std::cout<<"add"<<temp<<"to str";
		str += getSymbolTableIndex(temp);
		temp.clear();
	}

}

//get language
inline istream& operator>>(istream& is, mGLanguage& lan)
{
	lan.clear();
	string temp;
	int emptyCount = 0;

	while (!is.eof() && is.good())
	{
		temp.clear();
		getline(is, temp);

		mGString tempString;

		if (temp.length() == 0) /// skip empty line
		{
			emptyCount++;
			//cout << emptyCount;
			if (emptyCount == 2) // because by defualt takes in one
			{
				//	cout << "add";
				temp >> tempString;
				lan.disjoint_union(tempString);
			}
			continue;
		}

		temp >> tempString;
		lan += tempString;

	}
	return is;
}
