/*
    file: mGNFA.h 
    Project: miniGrail
    License: LGPLv2.1 http://www.gnu.org/licenses/lgpl-2.1.html

    Created by: by Rui Zhou
    Maintainer: Cezar Campeanu <ccampeanu@upei.ca>

*/

#ifndef _MGNFA
class mGNFA
{
protected:
	mGSet<mGState> startStates;
	mGSet<mGState> finalStates;
	mGSet<mGTransition> transitions;
	bool isAttached;
	mGMap<mGState, mGSet<mGTransition> > transitionsByState; // the dictionary
//in which transitions are attached to keys

//to rename states like "1 3 5" to "0 1 2"
	bool statesRenamed;
	mGArray<int> statesMapping;

public:
//constructor family
	mGNFA()
	{
		isAttached = false;
		statesRenamed = false;
	}
	mGNFA(const mGNFA& d) :
			startStates(d.startStates), finalStates(d.finalStates), transitions(
					d.transitions)
	{
	}
//~mGNFA(){}

//add a transition
	void addTransition(const mGTransition&);
//add a startState
	void addStart(const mGState&);
//add a finalState
	void addFinal(const mGState&);

//get methods
	inline const mGSet<mGState>& getStartStates() const
	{
		return startStates;
	}
	inline const mGSet<mGState>& getFinalStates() const
	{
		return finalStates;
	}
	inline const mGSet<mGTransition>& getTransitions() const
	{
		return transitions;
	}

	inline void clearTransitions()
	{

		transitions.clear();
	}

	//non cosnt overload
	//TODO necessary ?
	inline mGSet<mGState>& getStartStates()
	{
		return startStates;
	}
	inline mGSet<mGState>& getFinalStates()
	{
		return finalStates;
	}
	inline mGSet<mGTransition>& getTransitions()
	{
		return transitions;
	}

//question1012:  return mGSet is return by value? 
	inline const mGMap<mGState, mGSet<mGTransition> >& transitionsByStates() const
	{
		return transitionsByState;
	}

//gather the states
	void gatherStates(mGSet<mGState> &) const;

//gather the Labels
	void gatherLabels(mGSet<int> &) const;

//return one of the transition
	inline const mGTransition& operator[](int i) const
	{
		return transitions[i];
	}

	inline mGTransition& operator[](int i)
	{
		return transitions[i];
	}

//get the size of transitions
	inline int getTransitionSize() const
	{
		return transitions.size();
	}
//same interface as in Grail+
	inline int size() const
	{

		return transitions.size();
	}

//clear everything
	void clear();

//attach and de-attach the transitions to the states
	void attach();
	void attach_inv(mGMap<mGState, mGSet<mGTransition> >& );
	void clearAttach();

	void unqiueTransitionsByAttach();
	void writeBackFromAttached();

	mGSet<mGTransition>& getTransitionsStartFrom(const mGState&); //question no const?

//const mGSet<mGTransition>&  getTransitionsStartFrom( const mGState&)  const;//no const? 

	void rename(); ////to rename states like "1 3 5" to "0 1 2"

	void cancelRename(); //cancel previous renaming, if still meaningful

	//TODO : MORE ELEGENT WAY?
	int find_first(const mGState& r) const;
	/* from Grail+
	 * Description:
	 This function is a protected member of the template class fm (declared in
	 classes/re/fm.h).  If the set of arcs is sorted, this function can be
	 used to find the first occurrence of the given source state in the set
	 of arcs for this fm.  A recursive binary search is performed to do the
	 search.  Note that it is the programmer's responsibility to make sure
	 that the arcs are sorted before this function is called since no
	 checking is done.  The function returns the index of the first arc
	 containing the state r as its source, or -1 if no such arc is found.
	 */
	mGNFA& select(const mGState&, int, mGNFA&) const;
	// creates a sub NFA (third parameter) by selecting
	//  arcs from this fm with a given state (first
	//  parameter) as the source or sink state (depending
	//  on the value of the second parameter which can
	//  be SOURCE, SINK or EITHER).  The function
	//  returns a reference to the new nfa.

	mGSet<mGState>& sinks(mGSet<mGState>&) const;

	void reachable_states(mGSet<mGState>&);
	//This function is a public member of the template class fm (declared in
	//classes/re/fm.h).  This function extracts the set states reachable
	//from a start state in this fm and returns them as the parameter set, s.
	//This fm is unchanged.

	void reachable();
	// eliminate the unreachable states

	mGNFA& reverse();

	bool isCompleted();

};
#define _MGNFA
#endif
