/*
    file: mGTransition.h 
    Project: miniGrail
    License: LGPLv2.1 http://www.gnu.org/licenses/lgpl-2.1.html

    Created by: by Rui Zhou
    Maintainer: Cezar Campeanu <ccampeanu@upei.ca>


 This class is the transition
 Composed of two states and a label
 
 ___                  ___
 /   \       a        /   \       Change from state 1 to state
 | 1 |--------------> | 2 |       2 on input a.
 \___/                \___/

 */

#ifndef _MGTRANSITION

class mGTransition
{
protected:
	mGState start;
	int label; // short ????
	mGState end;

public:
	inline mGTransition()
	{
		label = NULL_INT;
	}
	mGTransition(const mGTransition & t) :
			start(t.start), label(t.label), end(t.end)
	{
	}
	mGTransition(int, int, int);
	mGTransition(int, string, int);
	~mGTransition()
	{
	}

//assign
	mGTransition& operator=(const mGTransition&);
//compare
	bool operator==(const mGTransition&) const;
	bool operator<(const mGTransition&) const;
	bool operator>(const mGTransition&) const;
//only compare start and label
//used when add transition to dfa
//we want to make sure we do not make a dfa to a nfa
	bool sameStartAndLabel(const mGTransition&) const;

//get and set

	inline mGTransition& assign(mGState& s, int l, mGState& e)
	{
		start = s;
		label = l;
		end = e;
		return *this;
	}

	inline mGState& getStart()
	{
		return start;
	} //question? why can not const?

	inline mGState& getEnd()
	{
		return end;
	}

	inline const mGState& getStart() const
	{
		return start;
	} //question? why can not const?
	inline const mGState& getEnd() const
	{
		return end;
	}

	inline int getLabel() const
	{

		return label;
	}
	inline void setStart(const mGState& s)
	{
		start = s;
	}
	inline void setEnd(const mGState& s)
	{
		end = s;
	}

	void setStart(int i);
	void setEnd(int i);

	inline void setLabel(const int l)
	{
		label = l;
	}
	void null();

	inline void reverse()
	{
		mGState temp = start;
		start = end;
		end = temp;

	}

	void getReverseCopy(mGTransition&);

};
#define _MGTRANSITION
#endif
